from tespy.components import *
from tespy.connections import Connection
from tespy.networks import Network
import CoolProp.CoolProp as CP
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd

def option_base_mode0(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_bypass = Merge('Merge Bypass')
    me_owd = Merge('Merge OWD')
    sp_tud_1 = Splitter('Splitter TUD 1')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in1', label='5')
    c7 = Connection(me_bypass, 'out1', cc, 'in1', label='7')
    c11 = Connection(cc, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')

    c21 = Connection(pu_ev, 'out1', he_dap, 'in2', label='21')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 

    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
    c16 = Connection(sp_tud_1, 'out2', va_bypass, 'in1', label='16')
    c17 = Connection(sp_tud_1, 'out1', me_bypass, 'in2', label='17')
    c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
    c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c7, c11, c12, c13, c14, c15, c16, c17, c19, c21, c25, c26, c27)

    he_dap.set_attr(pr1=0.98, pr2=0.98, ttd_l=2)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1)
    c5.set_attr(fluid={'water': 1}, T=74.5)
    c5.set_attr(p=1)
    c11.set_attr(T=t_supply+1)
    c15.set_attr(p=1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c17, c19, c27)

        c27 = Connection(tud, 'out1', me_tud, 'in1', label='27')
        c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
        c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
        c17 = Connection(me_tud, 'out1', me_bypass, 'in2', label='17')
        c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
        c19 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='19')
        nw.add_conns(c14, c15, c16, c17, c19, c27)
        c27.set_attr(T = t_return_tud+1)
        c14.set_attr(T = t_return_owd+1)
        c15.set_attr(p=1)
        nw.solve('design')
    
    return nw

def option_base_mode1(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_buffer = Splitter('Splitter Buffer')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    sp_wkc = Splitter('Splitter WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c10 = Connection(me_wkc, 'out1', me_buffer, 'in1', label='10')
    c11 = Connection(me_buffer, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c17 = Connection(me_tud, 'out1', sp_buffer, 'in1', label='17')
    c18 = Connection(sp_buffer, 'out2', me_bypass, 'in1', label='18')
    c19 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c22 = Connection(sp_buffer, 'out1', va_buffer, 'in1', label='22')
    c23 = Connection(va_buffer, 'out1', buffer, 'in1', label='23')
    c24 = Connection(buffer, 'out1', me_buffer, 'in2', label='24')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', me_tud, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c23, c24, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c19.set_attr(v=375, p=1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c24.set_attr(T=t_buffer)
    c4.set_attr(T=98) # limit temperature at output boiler to minimize flow through boiler and maximimze flow through buffer

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    if nw.results['Connection'].loc['6'].v < 0:

        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        nw.del_conns(c2, c5, c6, c7, c18)
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c18 = Connection(sp_buffer, 'out2', sp_bypass_1, 'in1', label='18')

        nw.add_conns(c2, c5, c6, c7, c18)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
        nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_base_mode2(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    sp_bypass_1 = Splitter('Splitter Bypass 1')
    va_bypass = Valve('Valve Bypass')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    me_wkc_1 = Merge('Merge WKC 1')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
    c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
    c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c11 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='11') 
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c19 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', me_tud, 'in1', label='27')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c17 = Connection(me_tud, 'out1', sp_bypass_1, 'in1', label='17')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c11, c12, c13, c14, c15, c16, c17, c19, c20, c21, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q = q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5)
    c6.set_attr(p=1)
    c19.set_attr(p=1)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c5.set_attr(v=375)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:

        me_bypass = Merge('Merge Bypass')
        sp_wkc = Splitter('Splitter WKC')

        nw.del_conns(c2, c5, c6, c7, c17)
        c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
        c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
        c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
        c17 = Connection(me_tud, 'out1', me_bypass, 'in1', label='17')
   
        nw.add_conns(c2, c5, c6, c7, c17)

        c5.set_attr(fluid={'water': 1}, T=74.5, v=375)
        c6.set_attr(p=1)
        nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_base_mode3(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6
    t_supply = t_supply
    t_return_tud = t_return_tud
    t_return_owd = t_return_owd

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    me_owd = Merge('Merge OWD')
    sp_buffer = Splitter('Splitter Buffer')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    sp_wkc = Splitter('Splitter WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c10 = Connection(me_wkc, 'out1', me_buffer, 'in1', label='10')
    c11 = Connection(me_buffer, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
    c16 = Connection(sp_tud_1, 'out2', va_bypass, 'in1', label='16')
    c17 = Connection(sp_tud_1, 'out1', sp_buffer, 'in1', label='17')
    c18 = Connection(sp_buffer, 'out2', me_bypass, 'in1', label='18')
    c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c22 = Connection(sp_buffer, 'out1', va_buffer, 'in1', label='22')
    c23 = Connection(va_buffer, 'out1', buffer, 'in1', label='23')
    c24 = Connection(buffer, 'out1', me_buffer, 'in2', label='24')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c23, c24, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q = q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c19.set_attr(p=1)
    c20.set_attr(v=375)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c24.set_attr(T=t_buffer)
    c4.set_attr(T=89) # this defines flow that has to go through boiler, rest is going through buffer

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    if nw.results['Connection'].loc['6'].v < 0:

        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        nw.del_conns(c2,c5,c6,c7,c18)
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c18 = Connection(sp_buffer, 'out2', sp_bypass_1, 'in1', label='18')

        nw.add_conns(c2,c5,c6,c7,c18)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
        nw.solve('design')      
    
    if nw.results['Connection'].loc['2'].v < 0:
        c4.set_attr(T=None)
        c4.set_attr(v=abs(nw.results['Connection'].loc['2'].v))
        nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_base_mode4(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6
    t_supply = t_supply
    t_return_tud = t_return_tud
    t_return_owd = t_return_owd

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    sp_wkc = Splitter('Splitter WKC')
    me_wkc = Merge('Merge WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c11 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='11') 
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', sp_tud1, 'in1', label='27')
    c16 = Connection(sp_tud1, 'out2', va_bypass, 'in1', label='16')
    c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
    c18 = Connection(sp_tud1, 'out1', me_bypass, 'in1', label='18')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c11, c12, c13, c14, c15, c16, c18, c19, c20, c21, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q = q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c19.set_attr(p=1)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    if nw.results['Connection'].loc['6'].v < 0:

        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        nw.del_conns(c2, c5, c6, c7, c18)
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c18 = Connection(sp_tud1, 'out1', sp_bypass_1, 'in1', label='18')

        nw.add_conns(c2, c5, c6, c7, c18)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
        nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_base_mode5(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6
    
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_bypass = Merge('Merge Bypass')
    me_owd = Merge('Merge OWD')
    va_bypass = Valve('Valve Bypass')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    sp_buffer = Splitter('Splitter Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')
    
    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in1', label='5')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c10= Connection(pu_tud, 'out1', sp_buffer, 'in1', label='10')
    c12 = Connection(sp_buffer, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
    c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
    c17 = Connection(sp_tud_1, 'out2', me_buffer, 'in1', label='17')
    c18 = Connection(me_buffer, 'out1', me_bypass, 'in2', label='18')    
    c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c22 = Connection(sp_buffer, 'out2', va_buffer, 'in1', label='22')
    c23 = Connection(va_buffer, 'out1', buffer, 'in1', label='23')
    c24 = Connection(buffer, 'out1', me_buffer, 'in2', label='24')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c7, c8, c9, c10, c12, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c23, c24, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q = q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5)
    c7.set_attr(p=1)
    c19.set_attr(p=1)
    c8.set_attr(T=t_supply+1)
    c20.set_attr(v=375)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer
    c24.set_attr(T=t_buffer)
    
    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_base_mode6(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_bypass = Merge('Merge Bypass')
    me_owd = Merge('Merge OWD')
    va_bypass = Valve('Valve Bypass')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')
    
    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in1', label='5')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c10 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='10')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
    c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
    c18 = Connection(sp_tud_1, 'out2', me_bypass, 'in2', label='18')    
    c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c7, c8, c9, c10, c13, c14, c15, c16, c18, c19, c20, c21, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, v=375)
    c7.set_attr(p=1)
    c19.set_attr(p=1)
    c8.set_attr(T=t_supply+1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)
  
    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    if nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c18, c19, c27)
        sp_owd = Splitter('Splitter OWD')
        me_tud = Merge('Merge TUD')

        c14 = Connection(owd, 'out1', sp_owd, 'in1', label='14')
        c15 = Connection(sp_owd, 'out2', va_bypass, 'in1', label='15')
        c16 = Connection(va_bypass, 'out1', me_tud, 'in1', label='16')
        c18 = Connection(me_tud, 'out1', me_bypass, 'in2', label='18')
        c19 = Connection(sp_owd, 'out1', pu_ev, 'in1', label='19')
        c27 = Connection(tud, 'out1', me_tud, 'in2', label='27')
        nw.add_conns(c14, c15, c16, c18, c19, c27)
        c19.set_attr(p=1)
        c27.set_attr(T = t_return_tud+1)
        c14.set_attr(T = t_return_owd+1)
        nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_base_mode7(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, t_buffer, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
   
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    me_tud = Merge('Merge TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    sp_buffer = Splitter('Splitter Buffer')
    va_buffer = Valve('Valve Buffer')
    
    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', cd, 'in2', label='5')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c10= Connection(pu_tud, 'out1', sp_buffer, 'in1', label='10')
    c12 = Connection(sp_buffer, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c16 = Connection(me_tud, 'out1', me_owd, 'in2', label='16')
    c19 = Connection(me_owd, 'out1',ev, 'in1', label='19')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c22 = Connection(sp_buffer, 'out2', va_buffer, 'in1', label='22')
    c23 = Connection(va_buffer, 'out1', buffer, 'in1', label='23')
    c24 = Connection(buffer, 'out1', me_tud, 'in2', label='24')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', me_tud, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit='m3 / h')
    nw.add_conns(c0, c1, c5, c8, c9, c10, c12, c13, c14, c16, c19, c21, c22, c23, c24, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    buffer.set_attr(pr=0.98)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c19.set_attr(v=375)
    c8.set_attr(T=t_supply+1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer
    c24.set_attr(T=t_buffer)
    
    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_base_mode8(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, v_sum, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
   
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', cd, 'in2', label='5')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c12= Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c19 = Connection(me_owd, 'out1', ev, 'in1', label='19')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c25 = Connection(sp_tud, 'out2', va_tud, 'in1', label='25')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', me_owd, 'in2', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c8, c9, c12, c13, c14, c19, c21, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=v_sum)
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    c0.set_attr(v=None)
    he_dap.set_attr(ttd_l=2)
    nw.solve('design')
    return nw

def option_1_mode1(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_bypass = Valve('Valve Bypass')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Buffer
    buffer = SimpleHeatExchanger('Buffer')
    me_buffer = Merge('Merge Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates = Splitter('Splitter ATES')
    sp_ates_1 = Splitter('Splitter ATES 1')
    me_ates = Merge('Merge ATES')

    # WKC
    me_wkc_1 = Merge('Merge WKC 1')
    me_wkc = Merge('Merge WKC')
    wkc = SimpleHeatExchanger('WKC')
    va_wkc = Valve('Valve WKC')
    sp_wkc = Splitter('Splitter WKC')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(me_wkc_1, 'out1', wkc, 'in1', label='2')
    c3 = Connection(wkc, 'out1', me_wkc, 'in1', label='3')
    c4 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='4')
    c5 = Connection(sp_ates, 'out1', me_wkc_1, 'in2', label='5')
    c6 = Connection(sp_ates, 'out2', va_wkc, 'in1', label='6')
    c8 = Connection(va_wkc, 'out1', cd, 'in2', label='8')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c10 = Connection(cc, 'out1', me_wkc, 'in2', label='10')
    c11 = Connection(me_wkc, 'out1', me_buffer, 'in1', label='11')
    c12 = Connection(me_buffer, 'out1', pu_tud, 'in1', label='12')
    c13 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c16 = Connection(sp_tud_1, 'out1', me_owd, 'in2', label='16')
    c17 = Connection(sp_tud_1, 'out2', va_bypass, 'in1', label='17')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', sp_ates_1, 'in1', label='22')
    c23 = Connection(sp_ates_1, 'out1', va_ates, 'in1', label='23')
    c31 = Connection(va_ates, 'out1', he_dap, 'in2', label='31')
    c25 = Connection(va_bypass, 'out1', buffer, 'in1', label='25')
    c26 = Connection(buffer, 'out1', me_buffer, 'in2', label='26')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c29 = Connection(tud, 'out1', sp_tud_1, 'in1', label='29')
    c30 = Connection(sp_ates_1, 'out2', he_ates, 'in2', label='30')
    c32 = Connection(he_ates, 'out2', sp_ates, 'in1', label='32')
    c33 = Connection(he_ates, 'out1', ates_cw, 'in1', label='33')
    c34 = Connection(ates_hw, 'out1', he_ates, 'in1', label='34')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c20, c21, c22, c23, c25, c26, c27, c28, c29, c30, c31, c32, c33, c34, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_l = 1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c4.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c9.set_attr(T=t_supply+1)
    c12.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c29.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c26.set_attr(T = t_buffer)
    c3.set_attr(T = 98) # limit temperature at output boiler to minimize flow through boiler and maximimze flow through buffer

    # ATES
    c34.set_attr(T=t_hw, fluid={'water': 1}, p=1)
    c32.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['5'].v < 0:

        nw.del_conns(c2, c4, c5, c6, c32)

        c2 = Connection(sp_wkc, 'out1', wkc, 'in1', label='2')
        c4 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='4')
        c5 = Connection(sp_wkc, 'out2', me_ates, 'in2', label='5')
        c6 = Connection(me_ates, 'out1', va_wkc, 'in1', label='6')  
        c32 = Connection(he_ates, 'out2', me_ates, 'in1', label='32')

        nw.add_conns(c2, c4, c5, c6, c32)
        c4.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
        c32.set_attr(T=t_hw-1)
        nw.solve('design')
        
    c44.set_attr(p=None, x=1, fluid={'NH3':1})
    ev.set_attr(ttd_l=5)
    nw.solve('design')

    return nw

def option_1_mode2(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates = Splitter('Splitter ATES')
    sp_ates_1 = Splitter('Splitter ATES 1')
    me_ates = Merge('Merge ATES')

    # WKC
    me_wkc_1 = Merge('Merge WKC 1')
    me_wkc = Merge('Merge WKC')
    wkc = SimpleHeatExchanger('WKC')
    va_wkc = Valve('Valve WKC')
    sp_wkc = Splitter('Splitter WKC')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(me_wkc_1, 'out1', wkc, 'in1', label='2')
    c3 = Connection(wkc, 'out1', me_wkc, 'in1', label='3')
    c4 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='4')
    c5 = Connection(sp_ates, 'out1', me_wkc_1, 'in2', label='5')
    c6 = Connection(sp_ates, 'out2', va_wkc, 'in1', label='6')
    c8 = Connection(va_wkc, 'out1', cd, 'in2', label='8')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c10 = Connection(cc, 'out1', me_wkc, 'in2', label='10')
    c12 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='12')
    c13 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c17 = Connection(tud, 'out1', me_owd, 'in2', label='17')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', sp_ates_1, 'in1', label='22')
    c23 = Connection(sp_ates_1, 'out1', va_ates, 'in1', label='23')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c30 = Connection(sp_ates_1, 'out2', he_ates, 'in2', label='30')
    c31 = Connection(va_ates, 'out1', he_dap, 'in2', label='31')
    c32 = Connection(he_ates, 'out2', sp_ates, 'in1', label='32')
    c33 = Connection(he_ates, 'out1', ates_cw, 'in1', label='33')
    c34 = Connection(ates_hw, 'out1', he_ates, 'in1', label='34')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c8, c9, c10, c12, c13, c14, c15, c17, c20, c21, c22, c23, c27, c28, c30, c31, c32, c33, c34, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_l = 1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(offdesign=["ttd_l"])
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c4.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c9.set_attr(T=t_supply+1)
    c12.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c17.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # ATES
    c23.set_attr(v=375)
    c34.set_attr(T=t_hw, fluid={'water': 1}, p=1)
    c32.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(design=["p"])
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    nw.solve('design')
    
    if nw.results['Connection'].loc['5'].v < 0:

        nw.del_conns(c2, c4, c5, c6, c32)

        c2 = Connection(sp_wkc, 'out1', wkc, 'in1', label='2')
        c4 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='4')
        c5 = Connection(sp_wkc, 'out2', me_ates, 'in2', label='5')
        c6 = Connection(me_ates, 'out1', va_wkc, 'in1', label='6')  
        c32 = Connection(he_ates, 'out2', me_ates, 'in1', label='32')

        nw.add_conns(c2, c4, c5, c6, c32)
        c4.set_attr(fluid={'water': 1}, T=74.5, p=1)
        c32.set_attr(T=t_hw-1)
        nw.solve('design')
    
    c44.set_attr(p=None, x=1, fluid={'NH3':1})
    ev.set_attr(ttd_l=5)
    nw.solve('design')
        
    return nw

def option_1_mode25(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6
    
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_bypass = Splitter('Splitter Bypass')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates = Splitter('Splitter ATES')
    sp_ates_1 = Splitter('Splitter ATES 1')
    me_ates = Merge('Merge ATES')

    # WKC
    me_wkc_1 = Merge('Merge WKC 1')
    me_wkc = Merge('Merge WKC')
    wkc = SimpleHeatExchanger('WKC')
    va_wkc = Valve('Valve WKC')
    sp_wkc = Splitter('Splitter WKC')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(me_wkc_1, 'out1', wkc, 'in1', label='2')
    c3 = Connection(wkc, 'out1', me_wkc, 'in1', label='3')
    c4 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='4')
    c5 = Connection(sp_ates, 'out1', me_wkc_1, 'in2', label='5')
    c6 = Connection(sp_ates, 'out2', va_wkc, 'in1', label='6')
    c7 = Connection(va_wkc, 'out1', me_bypass, 'in1', label='7')
    c8 = Connection(me_bypass, 'out1', cd, 'in2', label='8')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c10 = Connection(cc, 'out1', me_wkc, 'in2', label='10')
    c11 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='11')
    c13 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_owd, 'in2', label='16')
    c17 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='17')
    c18 = Connection(sp_tud_1, 'out2', me_bypass, 'in2', label='18')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', sp_ates_1, 'in1', label='22')
    c23 = Connection(sp_ates_1, 'out1', he_dap, 'in2', label='23')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c29 = Connection(tud, 'out1', sp_tud_1, 'in1', label='29')
    c30 = Connection(sp_ates_1, 'out2', va_ates, 'in1', label='30')
    c31 = Connection(va_ates, 'out1', he_ates, 'in2', label='31')
    c32 = Connection(he_ates, 'out2', sp_ates, 'in1', label='32')
    c33 = Connection(he_ates, 'out1', ates_cw, 'in1', label='33')
    c34 = Connection(ates_hw, 'out1', he_ates, 'in1', label='34')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c13, c14, c15, c16, c17, c18, c20, c21, c22, c23, c27, c28, c29, c30, c31, c32, c33, c34, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c4.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c9.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c29.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # ATES
    c31.set_attr(v=375)
    c34.set_attr(T=t_hw, fluid={'water': 1}, p=1) 
    c33.set_attr(v=375)
    c32.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        
        nw.del_conns(c5, c6, c7, c8, c18, c32)
        
        c5 = Connection(me_ates, 'out1', me_wkc_1, 'in2', label='5')
        c6 = Connection(va_wkc, 'out1', me_ates, 'in1', label='6')
        c7 = Connection(sp_bypass, 'out1', va_wkc, 'in1', label='7')
        c8 = Connection(sp_bypass, 'out2', cd, 'in2', label='8')    
        c18 = Connection(sp_tud_1, 'out2', sp_bypass, 'in1', label='18')
        c32 = Connection(he_ates, 'out2', me_ates, 'in2', label='32')

        nw.add_conns(c5, c6, c7, c8, c18, c32)
        c32.set_attr(T=t_hw-1)
        nw.solve('design')
        if nw.results['Connection'].loc['16'].v < 0:
            nw.del_conns(c15, c16, c17, c18, c20, c29)
            sp_owd = Splitter('Splitter OWD')
            me_tud = Merge('Merge TUD')

            c15 = Connection(owd, 'out1', sp_owd, 'in1', label='15')
            c16 = Connection(sp_owd, 'out2', va_bypass, 'in1', label='16')
            c17 = Connection(va_bypass, 'out1', me_tud, 'in1', label='17')
            c18 = Connection(me_tud, 'out1', sp_bypass, 'in1', label='18')
            c20 = Connection(sp_owd, 'out1', pu_ev, 'in1', label='20')
            c29 = Connection(tud, 'out1', me_tud, 'in2', label='29')
            nw.add_conns(c15, c16, c17, c18, c20, c29)
            c20.set_attr(p=1)
            c29.set_attr(T = t_return_tud+1)
            c15.set_attr(T = t_return_owd+1)
            nw.solve('design')

    if nw.results['Connection'].loc['5'].v < 0:
        nw.del_conns(c2, c4, c5, c6, c7, c32)

        c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
        c5a = Connection(va_wkc, 'out1', wkc, 'in1', label='5a')
        c4 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='4')
        c5 = Connection(sp_wkc, 'out2', me_ates, 'in2', label='5')
        c6 = Connection(me_ates, 'out1', me_bypass, 'in1', label='6')
        c32 = Connection(he_ates, 'out2', me_ates, 'in1', label='32')

        nw.add_conns(c2, c4, c5, c5a, c6, c32)
        c32.set_attr(T=t_hw-1)
        c4.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
        nw.solve('design')

    if nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c15, c16, c17, c18, c20, c29)
        sp_owd = Splitter('Splitter OWD')
        me_tud = Merge('Merge TUD')

        c15 = Connection(owd, 'out1', sp_owd, 'in1', label='15')
        c16 = Connection(sp_owd, 'out2', va_bypass, 'in1', label='16')
        c17 = Connection(va_bypass, 'out1', me_tud, 'in1', label='17')
        c18 = Connection(me_tud, 'out1', me_bypass, 'in2', label='18')
        c20 = Connection(sp_owd, 'out1', pu_ev, 'in1', label='20')
        c29 = Connection(tud, 'out1', me_tud, 'in2', label='29')
        nw.add_conns(c15, c16, c17, c18, c20, c29)
        c20.set_attr(p=1)
        c29.set_attr(T = t_return_tud+1)
        c15.set_attr(T = t_return_owd+1)
        nw.solve('design')

    c44.set_attr(p=None, x=1, fluid={'NH3':1})
    ev.set_attr(ttd_l=5)
    nw.solve('design')

    return nw

def option_1_mode3(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_buffer = Valve('Valve Buffer')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_bypass = Splitter('Splitter Bypass')
    sp_bypass_1 = Splitter('Splitter Bypass 1')
    me_owd = Merge('Merge OWD')
    me_tud = Merge('Merge TUD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Buffer
    buffer = SimpleHeatExchanger('Buffer')
    sp_buffer = Splitter('Splitter Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # WKC
    me_wkc_1 = Merge('Merge WKC 1')
    me_wkc = Merge('Merge WKC')
    wkc = SimpleHeatExchanger('WKC')
    va_wkc = Valve('Valve WKC')
    sp_wkc = Splitter('Splitter WKC')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c10 = Connection(me_wkc, 'out1', me_buffer, 'in1', label='10')
    c11 = Connection(me_buffer, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c17 = Connection(me_tud, 'out1', sp_buffer, 'in1', label='17')
    c18 = Connection(sp_buffer, 'out2', me_bypass, 'in1', label='18')
    c19 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c22 = Connection(sp_buffer, 'out1', va_buffer, 'in1', label='22')
    c25 = Connection(va_buffer, 'out1', buffer, 'in1', label='25')
    c24 = Connection(buffer, 'out1', me_buffer, 'in2', label='24')
    c23 = Connection(sp_tud, 'out2', va_tud, 'in1', label='23')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', me_tud, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c23, c24, c25, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c19.set_attr(v=375, p=1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c24.set_attr(T=t_buffer)
    c4.set_attr(T=98) # limit temperature at output boiler to minimize flow through boiler and maximimze flow through buffer

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['16'].v < 0:

        nw.del_conns(c14, c15, c16, c27, c17, c19)
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c17 = Connection(sp_tud_1, 'out2', sp_buffer, 'in1', label='17')
        c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
        c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')
        nw.add_conns(c14, c15, c16, c27, c17, c19)

        c19.set_attr(v=375, p=1)
        c27.set_attr(T = t_return_tud+1)
        c14.set_attr(T = t_return_owd+1)
        nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        
        nw.del_conns(c2, c5, c6, c7, c18)
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c18 = Connection(sp_buffer, 'out2', sp_bypass_1, 'in1', label='18')

        nw.add_conns(c2, c5, c6, c7, c18)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
        nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    
    return nw

def option_1_mode4(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_bypass = Splitter('Splitter Bypass')
    sp_bypass_1 = Splitter('Splitter Bypass 1')
    me_owd = Merge('Merge OWD')
    me_tud = Merge('Merge TUD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # WKC
    me_wkc_1 = Merge('Merge WKC 1')
    me_wkc = Merge('Merge WKC')
    wkc = SimpleHeatExchanger('WKC')
    va_wkc = Valve('Valve WKC')
    sp_wkc = Splitter('Splitter WKC')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c11 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c18 = Connection(me_tud, 'out1', me_bypass, 'in1', label='18')
    c19 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c23 = Connection(sp_tud, 'out2', va_tud, 'in1', label='23')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', me_tud, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c11, c12, c13, c14, c15, c16, c18, c19, c20, c21, c23, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c19.set_attr(v=375, p=1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        
        nw.del_conns(c2, c5, c6, c7, c18)
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c18 = Connection(me_tud, 'out1', sp_bypass_1, 'in1', label='18')

        nw.add_conns(c2, c5, c6, c7, c18)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
        nw.solve('design')

        if nw.results['Connection'].loc['16'].v < 0:

            nw.del_conns(c14, c15, c16, c27, c18, c19)
            c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
            c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
            c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
            c18 = Connection(sp_tud_1, 'out2', sp_bypass_1, 'in1', label='17')
            c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
            c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')
            nw.add_conns(c14, c15, c16, c27, c18, c19)

            c19.set_attr(v=375, p=1)
            c27.set_attr(T = t_return_tud+1)
            c14.set_attr(T = t_return_owd+1)
            nw.solve('design')
    
    elif nw.results['Connection'].loc['16'].v < 0:

        nw.del_conns(c14, c15, c16, c27, c18, c19)
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c18 = Connection(sp_tud_1, 'out2', me_bypass, 'in1', label='17')
        c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
        c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')
        nw.add_conns(c14, c15, c16, c27, c18, c19)

        c19.set_attr(v=375, p=1)
        c27.set_attr(T = t_return_tud+1)
        c14.set_attr(T = t_return_owd+1)
        nw.solve('design')
        
        if nw.results['Connection'].loc['6'].v < 0:
            
            nw.del_conns(c2, c5, c6, c7, c18)
            c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
            c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
            c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
            c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
            c18 = Connection(sp_tud_1, 'out2', sp_bypass_1, 'in1', label='17')


            nw.add_conns(c2, c5, c6, c7, c18)
            c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
            nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')       
    return nw

def option_1_mode5(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_buffer = Valve('Valve Buffer')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Buffer
    buffer = SimpleHeatExchanger('Buffer')
    sp_buffer = Splitter('Splitter Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates_1 = Splitter('Splitter ATES 1')
    me_ates = Merge('Merge ATES')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c4 = Connection(he_dap, 'out2', me_ates, 'in1', label='4')
    c5 = Connection(me_ates, 'out1', me_buffer, 'in1', label='5')
    c8 = Connection(me_buffer, 'out1', cd, 'in2', label='8')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c10 = Connection(cc, 'out1', pu_tud, 'in1', label='10')
    c11 = Connection(pu_tud, 'out1', sp_buffer, 'in1', label='11')
    c13 = Connection(sp_buffer, 'out2', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c24 = Connection(sp_buffer, 'out1', va_buffer, 'in1', label='24')
    c25 = Connection(va_buffer, 'out1', buffer, 'in1', label='25')
    c26 = Connection(buffer, 'out1', me_buffer, 'in2', label='26')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', sp_ates_1, 'in1', label='22')
    c23 = Connection(sp_ates_1, 'out1', va_ates, 'in1', label='23')
    c31 = Connection(va_ates, 'out1', he_dap, 'in2', label='31')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c29 = Connection(tud, 'out1', me_owd, 'in2', label='29')
    c30 = Connection(sp_ates_1, 'out2', he_ates, 'in2', label='30')
    c32 = Connection(he_ates, 'out2', me_ates, 'in2', label='32')
    c33 = Connection(he_ates, 'out1', ates_cw, 'in1', label='33')
    c34 = Connection(ates_hw, 'out1', he_ates, 'in1', label='34')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c4, c5, c8, c9, c10, c11, c13, c14, c15, c20, c21, c22, c23, c24, c25, c26, c27, c28, c29, c30, c31, c32, c33, c34, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_l = 1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c4.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c9.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c29.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c26.set_attr(T = t_buffer)

    # ATES
    c23.set_attr(v=375)
    c34.set_attr(T=t_hw, fluid={'water': 1}, p=1)
    c32.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    c44.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_1_mode6(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates_1 = Splitter('Splitter ATES 1')
    me_ates = Merge('Merge ATES')
    
    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c4 = Connection(he_dap, 'out2', me_ates, 'in1', label='4')
    c8 = Connection(me_ates, 'out1', cd, 'in2', label='8')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c10 = Connection(cc, 'out1', pu_tud, 'in1', label='10')
    c11 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='11')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c16 = Connection(tud, 'out1', me_owd, 'in2', label='16')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', sp_ates_1, 'in1', label='22')
    c23 = Connection(sp_ates_1, 'out1', va_ates, 'in1', label='23')
    c31 = Connection(va_ates, 'out1', he_dap, 'in2', label='31')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c30 = Connection(sp_ates_1, 'out2', he_ates, 'in2', label='30')
    c32 = Connection(he_ates, 'out2', me_ates, 'in2', label='32')
    c33 = Connection(he_ates, 'out1', ates_cw, 'in1', label='33')
    c34 = Connection(ates_hw, 'out1', he_ates, 'in1', label='34')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c4, c8, c9, c10, c11, c14, c15, c16, c20, c21, c22, c23, c27, c28, c30, c31, c32, c33, c34, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_l = 1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c4.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c9.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c16.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # ATES
    c4.set_attr(v=375)
    c34.set_attr(T=t_hw, fluid={'water': 1}, p=1)
    c32.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    c44.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_1_mode7(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_buffer = Valve('Valve Buffer')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_bypass = Splitter('Splitter Bypass')
    me_owd = Merge('Merge OWD')
    me_tud = Merge('Merge TUD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Buffer
    buffer = SimpleHeatExchanger('Buffer')
    sp_buffer = Splitter('Splitter Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c4 = Connection(he_dap, 'out2', me_bypass, 'in1', label='4')
    c8 = Connection(me_bypass, 'out1', cd, 'in2', label='8')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c10 = Connection(cc, 'out1', pu_tud, 'in1', label='10')
    c11 = Connection(pu_tud, 'out1', sp_buffer, 'in1', label='11')
    c24 = Connection(sp_buffer, 'out1', va_buffer, 'in1', label='24')
    c13 = Connection(sp_buffer, 'out2', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c16 = Connection(me_tud, 'out1', me_owd, 'in2', label='16')
    c17 = Connection(sp_bypass, 'out1', me_tud, 'in1', label='17')
    c18 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='18')
    c19 = Connection(va_bypass, 'out1', me_bypass, 'in2', label='19')
    c25 = Connection(va_buffer, 'out1', buffer, 'in1', label='25')
    c26 = Connection(buffer, 'out1', sp_bypass, 'in1', label='26')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c29 = Connection(tud, 'out1', me_tud, 'in2', label='29')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c4, c8, c9, c10, c11, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c24, c25, c26, c27, c28, c29, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c4.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c9.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c29.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c26.set_attr(T = t_buffer)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['17'].v < 0:
        nw.del_conns(c26, c29, c18, c17, c16)
        c16 = Connection(sp_tud_1, 'out1', me_owd, 'in2', label='16')
        c17 = Connection(sp_tud_1, 'out2', me_buffer, 'in1', label='17')
        c18 = Connection(me_buffer, 'out1', va_bypass, 'in1', label='18')
        c26 = Connection(buffer, 'out1', me_buffer, 'in2', label='26')
        c29 = Connection(tud, 'out1', sp_tud_1, 'in1', label='29')
        nw.add_conns(c26, c29, c18, c17, c16)
        c26.set_attr(T = t_buffer)
        c29.set_attr(T = t_return_tud+1)
        nw.solve('design')
        
    c44.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_1_mode8(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_bypass = Splitter('Splitter Bypass')
    me_owd = Merge('Merge OWD')
    me_tud = Merge('Merge TUD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud_1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')
    
    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in2', label='5')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c18 = Connection(me_tud, 'out1', me_bypass, 'in1', label='18')
    c19 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='19')
    c20 = Connection(pu_ev, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c23 = Connection(sp_tud, 'out2', va_tud, 'in1', label='23')
    c26 = Connection(va_tud, 'out1', tud, 'in1', label='26') 
    c27 = Connection(tud, 'out1', me_tud, 'in1', label='27')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c7, c8, c9, c12, c13, c14, c15, c16, c18, c19, c20, c21, c23, c26, c27, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c8.set_attr(T=t_supply+1)
    c19.set_attr(p=1)
    c27.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['16'].v < 0:

        nw.del_conns(c14, c15, c16, c27, c18, c19)
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c18 = Connection(sp_tud_1, 'out2', me_bypass, 'in1', label='18')
        c19 = Connection(me_owd, 'out1', pu_ev, 'in1', label='19')
        c27 = Connection(tud, 'out1', sp_tud_1, 'in1', label='27')
        nw.add_conns(c14, c15, c16, c27, c18, c19)

        c19.set_attr(p=1)
        c27.set_attr(T = t_return_tud+1)
        c14.set_attr(T = t_return_owd+1)
        nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_1_mode9(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, t_cw, v_sum, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    # ATES
    ates_cw = Source('ATES cold well')
    ates_hw = Sink('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates = Splitter('Splitter ATES')
    me_ates = Merge('Merge ATES')

    c0 = Connection(dap_so, 'out1', sp_ates, 'in1', label='0')
    c1 = Connection(sp_ates, 'out1', he_dap, 'in1', label='1')
    c30 = Connection(sp_ates, 'out2', va_ates, 'in1', label='30')
    c31 = Connection(va_ates, 'out1', he_ates, 'in1', label='31')
    c32 = Connection(he_ates, 'out1', me_ates, 'in1', label='32')
    c33 = Connection(ates_cw, 'out1', he_ates, 'in2', label='33')
    c34 = Connection(he_ates, 'out2', ates_hw, 'in1', label='34')
    c2 = Connection(he_dap, 'out1', me_ates, 'in2', label='2')
    c4 = Connection(me_ates, 'out1', dap_si, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', cd, 'in2', label='5')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c12 = Connection(cc, 'out1', pu_tud, 'in1', label='12')
    c13= Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c20 = Connection(me_owd, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c17 = Connection(tud, 'out1', me_owd, 'in2', label='17')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c4, c5, c9, c12, c13, c14, c15, c17, c20, c21, c27, c28, c30, c31, c32, c33, c34, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_u=1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, p=1, v=375, T=76.5) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c1.set_attr(v=v_sum)
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c9.set_attr(T=t_supply+1)
    c17.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # ATES
    c33.set_attr(T=t_cw, p=1, fluid={'water': 1})
    c32.set_attr(T=t_cw+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    c44.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_1_mode95(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, t_cw, v_sum, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_bypass = Merge('Merge Bypass')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    sp_tud1 = Splitter('Splitter TUD 1')
    va_tud = Valve('Valve TUD')
    va_bypass = Valve('Valve Bypass')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_eva = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # ATES
    ates_cw = Source('ATES cold well')
    ates_hw = Sink('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates = Splitter('Splitter ATES')
    me_ates = Merge('Merge ATES')

    c0 = Connection(dap_so, 'out1', sp_ates, 'in1', label='0')
    c1 = Connection(sp_ates, 'out1', he_dap, 'in1', label='1')
    c30 = Connection(sp_ates, 'out2', va_ates, 'in1', label='30')
    c31 = Connection(va_ates, 'out1', he_ates, 'in1', label='31')
    c32 = Connection(he_ates, 'out1', me_ates, 'in1', label='32')
    c33 = Connection(ates_cw, 'out1', he_ates, 'in2', label='33')
    c34 = Connection(he_ates, 'out2', ates_hw, 'in1', label='34')
    c2 = Connection(he_dap, 'out1', me_ates, 'in2', label='2')
    c4 = Connection(me_ates, 'out1', dap_si, 'in1', label='4')

    c5 = Connection(he_dap, 'out2', me_bypass, 'in1', label='5')
    c6 = Connection(me_bypass, 'out1', cc, 'in1', label='6')
    c12 = Connection(cc, 'out1', pu_tud, 'in1', label='12')
    c13 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_owd, 'in2', label='16')
    c17 = Connection(sp_tud1, 'out1', va_bypass, 'in1', label='17')
    c18 = Connection(sp_tud1, 'out2', me_bypass, 'in2', label='18')
    c19 = Connection(me_owd, 'out1', pu_eva, 'in1', label='19') 
    c20 = Connection(pu_eva, 'out1', he_dap, 'in2', label='20')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c29 = Connection(tud, 'out1', sp_tud1, 'in1', label='29')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c4, c5, c6, c12, c13, c14, c15, c16, c17, c18, c19, c20, c27, c28, c29, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_u=1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_tud.set_attr(eta_s = 0.75)
    pu_eva.set_attr(eta_s = 0.75)

    c0.set_attr(fluid={'water': 1}, p=1, v=375, T=76.5)
    c1.set_attr(v=v_sum)
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c13.set_attr(T=t_supply+1)
    c29.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)
    c19.set_attr(p=1)

    # ATES
    c33.set_attr(T=t_cw, p=1, fluid={'water': 1})
    c32.set_attr(T=t_cw+1)

    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')

    c1.set_attr(v=None)
    he_dap.set_attr(ttd_l=2)
    nw.solve('design')
    return nw

def option_1_mode10(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, v_sum, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
   
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', cd, 'in2', label='5')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c12 = Connection(cc, 'out1', pu_tud, 'in1', label='12')
    c13= Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c20 = Connection(me_owd, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c17 = Connection(tud, 'out1', me_owd, 'in2', label='17')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c9, c12, c13, c14, c15, c17, c20, c21, c27, c28, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, p=1, T=76.5, v=v_sum) 
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c9.set_attr(T=t_supply+1)
    c17.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    c44.set_attr(p=None)
    c0.set_attr(v=None)
    he_dap.set_attr(ttd_l=2)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    
    return nw

def option_2_mode1(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_buffer = Splitter('Splitter Buffer')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    sp_wkc = Splitter('Splitter WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')
    sp_tud_1 = Splitter('Splitter TUD 1')
    me_owd = Merge('Merge OWD')
    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c10 = Connection(me_wkc, 'out1', me_buffer, 'in1', label='10')
    c11 = Connection(me_buffer, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c17 = Connection(me_tud, 'out1', sp_buffer, 'in1', label='17')
    c18 = Connection(sp_buffer, 'out2', he_ates, 'in2', label='18')
    c19 = Connection(he_ates, 'out2', me_bypass, 'in1', label='19')
    c20 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c23 = Connection(sp_buffer, 'out1', va_buffer, 'in1', label='23')
    c24 = Connection(va_buffer, 'out1', buffer, 'in1', label='24')
    c25 = Connection(buffer, 'out1', me_buffer, 'in2', label='25')
    c26 = Connection(sp_tud, 'out2', va_tud, 'in1', label='26')
    c27 = Connection(va_tud, 'out1', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')
    # ATES
    c35 = Connection(ates_hw, 'out1', he_ates, 'in1', label='35')
    c36 = Connection(he_ates, 'out1', ates_cw, 'in1', label='36')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c23, c24, c25, c26, c27, c28, c30, c31, c32, c33, c34, c35, c36)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_l=1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c20.set_attr(p=1/0.98)
    c21.set_attr(v=375)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c25.set_attr(T=t_buffer)
    c4.set_attr(T=98) # limit temperature at output boiler to minimize flow through boiler and maximimze flow through buffer

    # ATES
    c35.set_attr(T=t_hw, fluid={'water': 1}, p=1)
    c19.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        nw.del_conns(c2, c5, c6, c7, c19)
        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c19 = Connection(he_ates, 'out2', sp_bypass_1, 'in1', label='19')
        
        nw.add_conns(c2, c5, c6, c7, c19)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
        c19.set_attr(T=t_hw-1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
        
        if nw.results['Connection'].loc['16'].v < 0:
            nw.del_conns(c14, c15, c16, c17, c20, c28)   
            c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
            c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
            c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
            c17 = Connection(sp_tud_1, 'out2', sp_buffer, 'in1', label='17')
            c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
            c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
            nw.add_conns(c14, c15, c16, c17, c20, c28)

            c14.set_attr(T = t_return_owd+1)
            c20.set_attr(p=1/0.98)
            c28.set_attr(T = t_return_tud+1)

            nw.set_attr(iterinfo=iterinfo)
            nw.solve('design')
            
    elif nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c17, c20, c28)
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c17 = Connection(sp_tud_1, 'out2', sp_buffer, 'in1', label='17')
        c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c17, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1/0.98)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_2_mode2(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_hw, flow_bypass, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    sp_wkc = Splitter('Splitter WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c11 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c18 = Connection(me_tud, 'out1', he_ates, 'in2', label='18')
    c19 = Connection(he_ates, 'out2', me_bypass, 'in1', label='19')
    c20 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c26 = Connection(sp_tud, 'out2', va_tud, 'in1', label='26')
    c27 = Connection(va_tud, 'out1', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')
    # ATES
    c35 = Connection(ates_hw, 'out1', he_ates, 'in1', label='35')
    c36 = Connection(he_ates, 'out1', ates_cw, 'in1', label='36')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c11, c12, c13, c14, c15, c16, c18, c19, c20, c21, c22, c26, c27, c28, c30, c31, c32, c33, c34, c35, c36)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c20.set_attr(p=1/0.98)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # ATES
    c35.set_attr(T=t_hw, fluid={'water': 1}, p=1, v=flow_bypass)
    c19.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        nw.del_conns(c2, c5, c6, c7, c19)
        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c19 = Connection(he_ates, 'out2', sp_bypass_1, 'in1', label='19')
        
        nw.add_conns(c2, c5, c6, c7, c19)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
        c19.set_attr(T=t_hw-1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
    
    elif nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c18, c20, c28)
        sp_tud_1 = Splitter('Splitter TUD 1')
        me_owd = Merge('Merge OWD')
 
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c18 = Connection(sp_tud_1, 'out2', he_ates, 'in2', label='18')
        c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c18, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1/0.98)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
    
    c34.set_attr(p=None)
    c35.set_attr(v=None)
    he_ates.set_attr(ttd_l=1)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_2_mode25(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    sp_wkc = Splitter('Splitter WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')

    sp_bypass_2 = Splitter('Splitter Bypass 2')
    va_bypass_2 = Valve('Valve Bypass 2')
    me_bypass_2 = Merge('Merge Bypass 2')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', me_bypass_2, 'in1', label='7')
    c39 = Connection(me_bypass_2, 'out1', cd, 'in2', label='39')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c11 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c17 = Connection(me_tud, 'out1', sp_bypass_2, 'in1', label='17')
    c18 = Connection(sp_bypass_2, 'out1', he_ates, 'in2', label='18')
    c19 = Connection(he_ates, 'out2', me_bypass, 'in1', label='19')
    c20 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c26 = Connection(sp_tud, 'out2', va_tud, 'in1', label='26')
    c27 = Connection(va_tud, 'out1', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')
    # ATES
    c35 = Connection(ates_hw, 'out1', he_ates, 'in1', label='35')
    c36 = Connection(he_ates, 'out1', ates_cw, 'in1', label='36')
    c37 = Connection(sp_bypass_2, 'out2', va_bypass_2, 'in1', label='37')
    c38 = Connection(va_bypass_2, 'out1', me_bypass_2, 'in2', label='38')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c11, c12, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c26, c27, c28, c30, c31, c32, c33, c34, c35, c36, c37, c38, c39)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c18.set_attr(v=375)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c20.set_attr(p=1/0.98)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # ATES
    c35.set_attr(T=t_hw, fluid={'water': 1}, p=1)
    c36.set_attr(v=375)
    c19.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        nw.del_conns(c2, c5, c6, c7, c19)
        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', me_bypass_2, 'in1', label='7')
        c19 = Connection(he_ates, 'out2', sp_bypass_1, 'in1', label='19')
        
        nw.add_conns(c2, c5, c6, c7, c19)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
        c19.set_attr(T=t_hw-1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
        
    if nw.results['Connection'].loc['7'].v < 0:
        nw.del_conns(c2, c5, c6, c7, c19, c38, c39)
        me_bypass_1 = Merge('Merge Bypass 1')
        sp_bypass_3 = Splitter('Splitter Bypass 3')
        me_wkc_1 = Merge('Merge WKC 1')
        
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(me_bypass_1, 'out1', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_3, 'out1', me_bypass_1, 'in1', label='7')
        c19 = Connection(he_ates, 'out2', me_bypass_1, 'in2', label='19')
        c38 = Connection(va_bypass_2, 'out1', sp_bypass_3, 'in1', label='38')
        c39 = Connection(sp_bypass_3, 'out2', cd, 'in2', label='39')

        nw.add_conns(c2, c5, c6, c7, c19, c38, c39)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
        c19.set_attr(T=t_hw-1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
    
    if nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c17, c20, c28)
        sp_tud_1 = Splitter('Splitter TUD 1')
        me_owd = Merge('Merge OWD')
 
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c17 = Connection(sp_tud_1, 'out2', sp_bypass_2, 'in1', label='17')
        c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c17, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1/0.98)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
        
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_2_mode3(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_buffer = Splitter('Splitter Buffer')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    sp_wkc = Splitter('Splitter WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c10 = Connection(me_wkc, 'out1', me_buffer, 'in1', label='10')
    c11 = Connection(me_buffer, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c17 = Connection(me_tud, 'out1', sp_buffer, 'in1', label='17')
    c18 = Connection(sp_buffer, 'out2', me_bypass, 'in1', label='18')
    c20 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c23 = Connection(sp_buffer, 'out1', va_buffer, 'in1', label='23')
    c24 = Connection(va_buffer, 'out1', buffer, 'in1', label='24')
    c25 = Connection(buffer, 'out1', me_buffer, 'in2', label='25')
    c26 = Connection(sp_tud, 'out2', va_tud, 'in1', label='26')
    c27 = Connection(va_tud, 'out1', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c18, c20, c21, c22, c23, c24, c25, c26, c27, c28, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c25.set_attr(T=t_buffer)
    c4.set_attr(T=98) # limit temperature at output boiler to minimize flow through boiler and maximimze flow through buffer

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        nw.del_conns(c2, c5, c6, c7, c18)
        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c18 = Connection(sp_buffer, 'out2', sp_bypass_1, 'in1', label='18')
        
        nw.add_conns(c2, c5, c6, c7, c18)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')

    elif nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c17, c20, c28)
        sp_tud_1 = Splitter('Splitter TUD 1')
        me_owd = Merge('Merge OWD')
 
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c17 = Connection(sp_tud_1, 'out2', sp_buffer, 'in1', label='17')
        c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c17, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')  

    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')   
    return nw

def option_2_mode4(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    me_wkc = Merge('Merge WKC')
    sp_wkc = Splitter('Splitter WKC')
    va_wkc = Valve('Valve WKC')
    wkc = SimpleHeatExchanger('WKC')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c2 = Connection(sp_wkc, 'out1', va_wkc, 'in1', label='2')
    c3 = Connection(va_wkc, 'out1', wkc, 'in1', label='3')
    c4 = Connection(wkc, 'out1', me_wkc, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', sp_wkc, 'in1', label='5')
    c6 = Connection(sp_wkc, 'out2', me_bypass, 'in2', label='6')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', me_wkc, 'in2', label='9')
    c11 = Connection(me_wkc, 'out1', pu_tud, 'in1', label='11')
    c12 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='12')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c18 = Connection(me_tud, 'out1', me_bypass, 'in1', label='18')
    c20 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c26 = Connection(sp_tud, 'out2', va_tud, 'in1', label='26')
    c27 = Connection(va_tud, 'out1', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c11, c12, c13, c14, c15, c16, c18, c20, c21, c22, c26, c27, c28, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    wkc.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c8.set_attr(T=t_supply+1)
    c11.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['6'].v < 0:
        nw.del_conns(c2, c5, c6, c7, c18)
        sp_bypass_1 = Splitter('Splitter Bypass 1')
        me_wkc_1 = Merge('Merge WKC 1')
        
        c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
        c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
        c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
        c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
        c18 = Connection(me_tud, 'out1', sp_bypass_1, 'in1', label='18')

        nw.add_conns(c2, c5, c6, c7, c18)
        c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
        
        if nw.results['Connection'].loc['16'].v < 0:
            nw.del_conns(c14, c15, c16, c18, c20, c28)
            sp_tud_1 = Splitter('Splitter TUD 1')
            me_owd = Merge('Merge OWD')
    
            c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
            c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
            c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
            c18 = Connection(sp_tud_1, 'out2', sp_bypass_1, 'in1', label='18')
            c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
            c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
            nw.add_conns(c14, c15, c16, c18, c20, c28)

            c14.set_attr(T = t_return_owd+1)
            c20.set_attr(p=1)
            c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')

    elif nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c18, c20, c28)
        sp_tud_1 = Splitter('Splitter TUD 1')
        me_owd = Merge('Merge OWD')

        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c18 = Connection(sp_tud_1, 'out2', me_bypass, 'in1', label='18')
        c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c18, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')

        if nw.results['Connection'].loc['6'].v < 0:
            nw.del_conns(c2, c5, c6, c7, c18)
            sp_bypass_1 = Splitter('Splitter Bypass 1')
            me_wkc_1 = Merge('Merge WKC 1')
            
            c2 = Connection(me_wkc_1, 'out1', va_wkc, 'in1', label='2')
            c5 = Connection(he_dap, 'out2', me_wkc_1, 'in1', label='5')
            c6 = Connection(sp_bypass_1, 'out2', me_wkc_1, 'in2', label='6')
            c7 = Connection(sp_bypass_1, 'out1', cd, 'in2', label='7')
            c18 = Connection(sp_tud_1, 'out2', sp_bypass_1, 'in1', label='18')

            nw.add_conns(c2, c5, c6, c7, c18)
            c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)

            nw.set_attr(iterinfo=iterinfo)
            nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_2_mode5(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, t_hw, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_buffer = Splitter('Splitter Buffer')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    sp_tud_1 = Splitter('Splitter TUD 1')
    me_owd = Merge('Merge OWD')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in2', label='5')
    c6 = Connection(me_bypass, 'out1', me_buffer, 'in1', label='6')
    c7 = Connection(me_buffer, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c10 = Connection(pu_tud, 'out1', sp_buffer, 'in1', label='10')
    c11 = Connection(sp_buffer, 'out1', sp_tud, 'in1', label='11')
    c12 = Connection(sp_tud, 'out1', va_tud, 'in1', label='12')
    c13 = Connection(va_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c15 = Connection(sp_tud_1, 'out2', me_owd, 'in2', label='15')
    c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
    c18 = Connection(va_bypass, 'out1', he_ates, 'in2', label='18')
    c19 = Connection(he_ates, 'out2', me_bypass, 'in1', label='19')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c23 = Connection(sp_buffer, 'out2', va_buffer, 'in1', label='23')
    c24 = Connection(va_buffer, 'out1', buffer, 'in1', label='24')
    c25 = Connection(buffer, 'out1', me_buffer, 'in2', label='25')
    c27 = Connection(sp_tud, 'out2', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')

    # ATES
    c35 = Connection(ates_hw, 'out1', he_ates, 'in1', label='35')
    c36 = Connection(he_ates, 'out1', ates_cw, 'in1', label='36')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c18, c19, c20, c21, c22, c23, c24, c25, c27, c28, c30, c31, c32, c33, c34, c35, c36)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_l=1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c20.set_attr(p=1/0.98)
    c21.set_attr(v=375)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c25.set_attr(T=t_buffer)

    # ATES
    c35.set_attr(T=t_hw, fluid={'water': 1}, p=1)
    c19.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['15'].v < 0:
        nw.del_conns(c14, c15, c16, c18, c20, c28)
        c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
        c15 = Connection(sp_bypass, 'out2', me_tud, 'in2', label='15')
        c18 = Connection(me_tud, 'out1', he_ates, 'in2', label='18')
        c16 = Connection(sp_bypass, 'out1', va_bypass, 'in1', label='16')
        c20 = Connection(va_bypass, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c18, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1/0.98)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_2_mode6(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, t_hw, flow_bypass, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # ATES
    ates_cw = Sink('ATES cold well')
    ates_hw = Source('ATES hot well')
    he_ates = HeatExchanger('HE ATES')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in2', label='5')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c11 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='11')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c18 = Connection(me_tud, 'out1', he_ates, 'in2', label='18')
    c19 = Connection(he_ates, 'out2', me_bypass, 'in1', label='19')
    c20 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c26 = Connection(sp_tud, 'out2', va_tud, 'in1', label='26')
    c27 = Connection(va_tud, 'out1', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')
    # ATES
    c35 = Connection(ates_hw, 'out1', he_ates, 'in1', label='35')
    c36 = Connection(he_ates, 'out1', ates_cw, 'in1', label='36')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c7, c8, c9, c11, c13, c14, c15, c16, c18, c19, c20, c21, c22, c26, c27, c28, c30, c31, c32, c33, c34, c35, c36)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c8.set_attr(T=t_supply+1)
    c20.set_attr(p=1/0.98)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # ATES
    c35.set_attr(T=t_hw, fluid={'water': 1}, p=1, v=flow_bypass)
    c19.set_attr(T=t_hw-1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c18, c20, c28)
        sp_tud_1 = Splitter('Splitter TUD 1')
        me_owd = Merge('Merge OWD')
 
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c18 = Connection(sp_tud_1, 'out2', he_ates, 'in2', label='18')
        c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c18, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1/0.98)
        c28.set_attr(T = t_return_tud+1)
        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
        
    c34.set_attr(p=None)
    c35.set_attr(v=None)
    he_ates.set_attr(ttd_l=1)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    
    return nw

def option_2_mode7(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, q_condenser, t_buffer, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
    q_condenser = q_condenser*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_buffer = Splitter('Splitter Buffer')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')
    sp_tud_1 = Splitter('Splitter TUD 1')
    me_owd = Merge('Merge OWD')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    buffer = SimpleHeatExchanger('Buffer')
    me_buffer = Merge('Merge Buffer')
    va_buffer = Valve('Valve Buffer')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in2', label='5')
    c6 = Connection(me_bypass, 'out1', me_buffer, 'in1', label='6')
    c7 = Connection(me_buffer, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c10 = Connection(pu_tud, 'out1', sp_buffer, 'in1', label='10')
    c11 = Connection(sp_buffer, 'out1', sp_tud, 'in1', label='11')
    c12 = Connection(sp_tud, 'out1', va_tud, 'in1', label='12')
    c13 = Connection(va_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
    c15 = Connection(sp_tud_1, 'out2', me_owd, 'in2', label='15')
    c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
    c18 = Connection(va_bypass, 'out1', me_bypass, 'in1', label='18')
    c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c23 = Connection(sp_buffer, 'out2', va_buffer, 'in1', label='23')
    c24 = Connection(va_buffer, 'out1', buffer, 'in1', label='24')
    c25 = Connection(buffer, 'out1', me_buffer, 'in2', label='25')
    c27 = Connection(sp_tud, 'out2', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c18, c20, c21, c22, c23, c24, c25, c27, c28, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    buffer.set_attr(pr=0.98)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98, Q=q_condenser)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c8.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c21.set_attr(v=375)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Buffer: set T_out
    c25.set_attr(T=t_buffer)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['15'].v < 0:
        nw.del_conns(c14, c15, c16, c18, c20, c28)
        c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
        c15 = Connection(sp_bypass, 'out2', me_tud, 'in2', label='15')
        c18 = Connection(me_tud, 'out1', me_bypass, 'in1', label='18')
        c16 = Connection(sp_bypass, 'out1', va_bypass, 'in1', label='16')
        c20 = Connection(va_bypass, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c18, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_2_mode8(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, iterinfo=False):
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6

    dap_si = Sink('Sink')
    dap_so = Source('Source')
    sp_bypass = Splitter('Splitter Bypass')
    va_bypass = Valve('Valve Bypass')
    me_bypass = Merge('Merge Bypass')
    sp_tud = Splitter('Splitter TUD')
    me_tud = Merge('Merge TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    pu_ev = Pump('Pump before Evaporator')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', me_bypass, 'in2', label='5')
    c7 = Connection(me_bypass, 'out1', cd, 'in2', label='7')
    c8 = Connection(cd, 'out2', cc, 'in1', label='8')
    c9 = Connection(cc, 'out1', pu_tud, 'in1', label='9')
    c11 = Connection(pu_tud, 'out1', sp_tud, 'in1', label='11')
    c13 = Connection(sp_tud, 'out1', owd, 'in1', label='13')
    c14 = Connection(owd, 'out1', sp_bypass, 'in1', label='14')
    c15 = Connection(sp_bypass, 'out2', va_bypass, 'in1', label='15')
    c16 = Connection(va_bypass, 'out1', me_tud, 'in2', label='16')
    c18 = Connection(me_tud, 'out1', me_bypass, 'in1', label='18')
    c20 = Connection(sp_bypass, 'out1', pu_ev, 'in1', label='20')
    c21 = Connection(pu_ev, 'out1', ev, 'in1', label='21')
    c22 = Connection(ev, 'out1', he_dap, 'in2', label='22')
    c26 = Connection(sp_tud, 'out2', va_tud, 'in1', label='26')
    c27 = Connection(va_tud, 'out1', tud, 'in1', label='27') 
    c28 = Connection(tud, 'out1', me_tud, 'in1', label='28')

    # HP
    c30 = Connection(cc_hp, 'out1', cp, 'in1', label='30')
    c31 = Connection(cp, 'out1', cd, 'in1', label='31')
    c32 = Connection(cd, 'out1', va_hp, 'in1', label='32')
    c33 = Connection(va_hp, 'out1', ev, 'in2', label='33')
    c34 = Connection(ev, 'out2', cc_hp, 'in1', label='34')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c7, c8, c9, c11, c13, c14, c15, c16, c18, c20, c21, c22, c26, c27, c28, c30, c31, c32, c33, c34)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_ev.set_attr(eta_s=0.75)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, T=76.5, p=1, v=375) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1, v=375)
    c8.set_attr(T=t_supply+1)
    c20.set_attr(p=1)
    c28.set_attr(T = t_return_tud+1)
    c14.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c32.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c34.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    if nw.results['Connection'].loc['16'].v < 0:
        nw.del_conns(c14, c15, c16, c18, c20, c28)
        sp_tud_1 = Splitter('Splitter TUD 1')
        me_owd = Merge('Merge OWD')
 
        c14 = Connection(owd, 'out1', me_owd, 'in1', label='14')
        c15 = Connection(va_bypass, 'out1', me_owd, 'in2', label='15')
        c16 = Connection(sp_tud_1, 'out1', va_bypass, 'in1', label='16')
        c18 = Connection(sp_tud_1, 'out2', me_bypass, 'in1', label='18')
        c20 = Connection(me_owd, 'out1', pu_ev, 'in1', label='20')
        c28 = Connection(tud, 'out1', sp_tud_1, 'in1', label='28')
        nw.add_conns(c14, c15, c16, c18, c20, c28)

        c14.set_attr(T = t_return_owd+1)
        c20.set_attr(p=1)
        c28.set_attr(T = t_return_tud+1)

        nw.set_attr(iterinfo=iterinfo)
        nw.solve('design')
    
    c34.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    return nw

def option_2_mode9(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, t_cw, v_sum, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
   
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # ATES
    ates_cw = Source('ATES cold well')
    ates_hw = Sink('ATES hot well')
    he_ates = HeatExchanger('HE ATES')
    va_ates = Valve('Valve ATES')
    sp_ates = Splitter('Splitter ATES')
    me_ates = Merge('Merge ATES')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', sp_ates, 'in1', label='0')
    c1 = Connection(sp_ates, 'out1', he_dap, 'in1', label='1')
    c30 = Connection(sp_ates, 'out2', va_ates, 'in1', label='30')
    c31 = Connection(va_ates, 'out1', he_ates, 'in1', label='31')
    c32 = Connection(he_ates, 'out1', me_ates, 'in1', label='32')
    c33 = Connection(ates_cw, 'out1', he_ates, 'in2', label='33')
    c34 = Connection(he_ates, 'out2', ates_hw, 'in1', label='34')
    c2 = Connection(he_dap, 'out1', me_ates, 'in2', label='2')
    c4 = Connection(me_ates, 'out1', dap_si, 'in1', label='4')
    c5 = Connection(he_dap, 'out2', cd, 'in2', label='5')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c12 = Connection(cc, 'out1', pu_tud, 'in1', label='12')
    c13= Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c20 = Connection(me_owd, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c17 = Connection(tud, 'out1', me_owd, 'in2', label='17')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c2, c4, c5, c9, c12, c13, c14, c15, c17, c20, c21, c27, c28, c30, c31, c32, c33, c34, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    he_ates.set_attr(pr1=0.98, pr2=0.98, ttd_u=1)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, p=1, v=375, T=76.5) # 375m³/h equals to 400m³/h with density and heat capacity of geo fluid
    c1.set_attr(v=v_sum)
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c9.set_attr(T=t_supply+1)
    c17.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # ATES
    c33.set_attr(T=t_cw, p=1, fluid={'water': 1})
    c32.set_attr(T=t_cw+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    c44.set_attr(p=None)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    
    return nw

def option_2_mode10(q_owd, q_tud, t_supply, t_return_tud, t_return_owd, v_sum, iterinfo=False): 
    demand_owd = q_owd*-1e6
    demand_tud = q_tud*-1e6
   
    dap_si = Sink('Sink')
    dap_so = Source('Source')
    me_owd = Merge('Merge OWD')
    sp_tud = Splitter('Splitter TUD')
    va_tud = Valve('Valve TUD')
    cc = CycleCloser('CC')
    pu_tud = Pump('Pump before TUD')
    he_dap = HeatExchanger('DAP')

    tud = SimpleHeatExchanger('TUD')
    owd = SimpleHeatExchanger('OWD')

    # Heat Pump
    cd = Condenser('Condenser')
    va_hp = Valve('Expansion Valve')
    cp = Compressor('Compressor')
    ev = HeatExchanger('Evaporator')
    cc_hp = CycleCloser('CC Heat Pump')

    c0 = Connection(dap_so, 'out1', he_dap, 'in1', label='0')
    c1 = Connection(he_dap, 'out1', dap_si, 'in1', label='1')
    c5 = Connection(he_dap, 'out2', cd, 'in2', label='5')
    c9 = Connection(cd, 'out2', cc, 'in1', label='9')
    c12 = Connection(cc, 'out1', pu_tud, 'in1', label='12')
    c13= Connection(pu_tud, 'out1', sp_tud, 'in1', label='13')
    c14 = Connection(sp_tud, 'out1', owd, 'in1', label='14')
    c15 = Connection(owd, 'out1', me_owd, 'in1', label='15')
    c20 = Connection(me_owd, 'out1', ev, 'in1', label='20')
    c21 = Connection(ev, 'out1', he_dap, 'in2', label='21')
    c27 = Connection(sp_tud, 'out2', va_tud, 'in1', label='27')
    c28 = Connection(va_tud, 'out1', tud, 'in1', label='28') 
    c17 = Connection(tud, 'out1', me_owd, 'in2', label='17')

    # HP
    c40 = Connection(cc_hp, 'out1', cp, 'in1', label='40')
    c41 = Connection(cp, 'out1', cd, 'in1', label='41')
    c42 = Connection(cd, 'out1', va_hp, 'in1', label='42')
    c43 = Connection(va_hp, 'out1', ev, 'in2', label='43')
    c44 = Connection(ev, 'out2', cc_hp, 'in1', label='44')

    nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg', m_unit = 'kg / s', v_unit = 'm3 / h')
    nw.add_conns(c0, c1, c5, c9, c12, c13, c14, c15, c17, c20, c21, c27, c28, c40, c41, c42, c43, c44)

    he_dap.set_attr(pr1=0.98, pr2=0.98)
    tud.set_attr(pr=0.98, Q = demand_tud)
    owd.set_attr(pr=0.98, Q = demand_owd)
    pu_tud.set_attr(eta_s=0.75)

    # Heat Pump
    cd.set_attr(pr1=0.98, pr2=0.98)
    ev.set_attr(pr1=0.98, pr2=0.98)
    cp.set_attr(eta_s=0.85)

    c0.set_attr(fluid={'water': 1}, p=1, T=76.5, v=v_sum) 
    c5.set_attr(fluid={'water': 1}, T=74.5, p=1)
    c9.set_attr(T=t_supply+1)
    c17.set_attr(T = t_return_tud+1)
    c15.set_attr(T = t_return_owd+1)

    # Heat Pump
    # condensation point
    p_cond = CP.PropsSI("P", "Q", 0, "T", t_supply + 5 + 273.15, 'NH3') * 1e-5
    c42.set_attr(p=p_cond)
    # evaporation point
    p_eva = CP.PropsSI("P", "Q", 1, "T", 16 + 273.15, 'NH3') * 1e-5
    c44.set_attr(p=p_eva, x=1, fluid={'NH3':1})
    
    nw.set_attr(iterinfo=iterinfo)
    nw.solve('design')
    
    c44.set_attr(p=None)
    c0.set_attr(v=None)
    he_dap.set_attr(ttd_l=2)
    ev.set_attr(ttd_l=5)
    nw.solve('design')
    
    return nw



